#define ID_ENTRY 0x9090
#define ID_EXIT 0xF0F0
#define PAGEWRITE 0xA0A0

volatile Uint16* const ADDR_55 = (Uint16*)0x2200AAAA; /* The flash sees this as 0x5555 */
volatile Uint16* const ADDR_AA = (Uint16*)0x22005554; /* The flash sees this as 0xAAAA */
volatile Uint16* const ADDR_VENDOR = (Uint16*)0x22000000;
volatile Uint16* const ADDR_DEVICE = (Uint16*)0x22000002; /* The flash sees this as 0x0001 */

void flash_wait(void) {
  int i;
  volatile int j = 0; /* dummy volatile variable, just to keep the compiler honest... there's definitely some better way. */

  for(i = 0; i < 2000; i++) {
    j++;
  }
  return;
}

void flash_unlock(void) {
  *ADDR_55 = 0xAAAA;
  *ADDR_AA = 0x5555;
  return;
}

void flash_command(Uint16 command) {
  flash_unlock();
  *ADDR_55 = command;
  return;
}

Uint16 flash_vendorid(void) {
  Uint16 vendorid;
  flash_command(ID_ENTRY);
  slSynch();
  slSynch(); /* Bodge to wait for flash to complete the command */
  vendorid = *ADDR_VENDOR;
  flash_command(ID_EXIT);
  slSynch();
  slSynch();
  return vendorid;
}

Uint16 flash_devid(void) {
  Uint16 devid;
  flash_command(ID_ENTRY);
  slSynch();
  slSynch();
  devid = *ADDR_DEVICE;
  flash_command(ID_EXIT);
  slSynch();
  slSynch();
  return devid;
}

void flash_erase(volatile Uint16* dest) {
  int i;
  flash_command(PAGEWRITE);   
  for(i = 0; i < 128; i++) {
    *dest = 0xFFFF;
    dest++;
  }

  /* Wait for the write to finish... as above, horrible bodge but it seems to work just great for now */
  slSynch();
  slSynch();
  return;
}

void flash_write(volatile Uint16* source, volatile Uint16* dest) {
  int i;
  volatile Uint16* now = 0;
  flash_command(PAGEWRITE);

  for(i = 0; i < 128; i++) {
    *dest = *source;
    dest++;
    source++;
  }
  
  /* Wait for the write to finish... */
  slSynch();
  slSynch();
  return;
}
